@@ -1,15 +1,21 @@
 #' Row-wise tibble creation
 #'
+#' @description
+#' \lifecycle{maturing}
+#'
 #' Create [tibble]s using an easier to read row-by-row layout.
 #' This is useful for small tables of data where readability is
 #' important.  Please see \link{tibble-package} for a general introduction.
 #'
-#' `frame_data()` is an older name for `tribble()`. It will eventually
-#' be phased out.
-#'
-#' @param ... Arguments specifying the structure of a `tibble`.
+#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
+#'   Arguments specifying the structure of a `tibble`.
 #'   Variable names should be formulas, and may only appear before the data.
+#'   These arguments are processed with [rlang::list2()]
+#'   and support unquote via [`!!`] and unquote-splice via [`!!!`].
 #' @return A [tibble].
+#' @seealso
+#'   See [quasiquotation] for more details on tidy dots semantics,
+#'   i.e. exactly how  the `...` argument is processed.
 #' @export
 #' @examples
 #' tribble(
@@ -31,21 +37,25 @@ tribble <- function(...) {
   turn_frame_data_into_tibble(data$frame_names, data$frame_rest)
 }
 
-#' @export
-#' @rdname tribble
-#' @usage NULL
-frame_data <- tribble
-
 #' Row-wise matrix creation
 #'
+#' @description
+#' \lifecycle{maturing}
+#'
 #' Create matrices laying out the data in rows, similar to
 #' `matrix(..., byrow = TRUE)`, with a nicer-to-read syntax.
 #' This is useful for small matrices, e.g. covariance matrices, where readability
 #' is important. The syntax is inspired by [tribble()].
 #'
-#' @param ... Arguments specifying the structure of a `frame_matrix`.
+#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
+#'   Arguments specifying the structure of a `frame_matrix`.
 #'   Column names should be formulas, and may only appear before the data.
+#'   These arguments are processed with [rlang::list2()]
+#'   and support unquote via [`!!`] and unquote-splice via [`!!!`].
 #' @return A [matrix].
+#' @seealso
+#'   See [quasiquotation] for more details on tidy dots semantics,
+#'   i.e. exactly how  the `...` argument is processed.
 #' @export
 #' @examples
 #' frame_matrix(
@@ -59,20 +69,20 @@ frame_matrix <- function(...) {
 }
 
 extract_frame_data_from_dots <- function(...) {
-  dots <- dots_list(...)
+  dots <- list2(...)
 
   # Extract the names.
   frame_names <- extract_frame_names_from_dots(dots)
 
   # Extract the data
   if (length(frame_names) == 0 && length(dots) != 0) {
-    stopc("Expected at least one column name; e.g. `~name`")
+    cnd_signal(error_tribble_needs_columns())
   }
   frame_rest <- dots[-seq_along(frame_names)]
   if (length(frame_rest) == 0L) {
     # Can't decide on type in absence of data -- use logical which is
     # coercible to all types
-    frame_rest <- logical()
+    frame_rest <- unspecified()
   }
 
   validate_rectangular_shape(frame_names, frame_rest)
@@ -94,18 +104,15 @@ extract_frame_names_from_dots <- function(dots) {
     }
 
     if (length(el) != 2) {
-      stopc("Expected a column name with a single argument; e.g. `~name`")
+      cnd_signal(error_tribble_lhs_column_syntax(el[[2]]))
     }
 
     candidate <- el[[2]]
     if (!(is.symbol(candidate) || is.character(candidate))) {
-      stopc(
-        "Expected a symbol or string denoting a column name, not ",
-        friendly_type(type_of(candidate))
-      )
+      cnd_signal(error_tribble_rhs_column_syntax(candidate))
     }
 
-    frame_names <- c(frame_names, as.character(el[[2]]))
+    frame_names <- c(frame_names, as.character(candidate))
   }
 
   frame_names
@@ -118,42 +125,48 @@ validate_rectangular_shape <- function(frame_names, frame_rest) {
   # and validate that the supplied formula produces a rectangular
   # structure.
   if (length(frame_rest) %% length(frame_names) != 0) {
-    stopc(
-      sprintf(
-        "invalid specification: had %s elements and %s columns",
-        length(frame_rest),
-        length(frame_names)
-      )
-    )
+    cnd_signal(error_tribble_non_rectangular(
+      length(frame_names),
+      length(frame_rest)
+    ))
   }
 }
 
 turn_frame_data_into_tibble <- function(names, rest) {
-  frame_mat <- matrix(rest, ncol = length(names), byrow = TRUE)
+  if (is_empty(names)) return(new_tibble(list(), nrow = 0))
+
+  nrow <- length(rest) / length(names)
+  dim(rest) <- c(length(names), nrow)
+  dimnames(rest) <- list(names, NULL)
+  frame_mat <- t(rest)
   frame_col <- turn_matrix_into_column_list(frame_mat)
 
-  # Create a tbl_df and return it
-  names(frame_col) <- names
-  new_tibble(frame_col)
+  new_tibble(frame_col, nrow = nrow)
 }
 
 turn_matrix_into_column_list <- function(frame_mat) {
   frame_col <- vector("list", length = ncol(frame_mat))
+  names(frame_col) <- colnames(frame_mat)
+
   # if a frame_mat's col is a list column, keep it unchanged (does not unlist)
   for (i in seq_len(ncol(frame_mat))) {
     col <- frame_mat[, i]
-    if (some(col, needs_list_col) || !inherits(col, "list")) {
-      frame_col[[i]] <- col
-    } else {
-      frame_col[[i]] <- invoke(c, col)
+
+    if (!some(col, needs_list_col) && inherits(col, "list")) {
+      # Assign names for somewhat nice error message, remove later
+      names(col) <- rep_along(col, names(frame_col)[[i]])
+      col <- unname(vec_c(!!! col))
     }
+
+    frame_col[[i]] <- unname(col)
   }
   return(frame_col)
 }
 
 turn_frame_data_into_frame_matrix <- function(names, rest) {
-  if (some(rest, needs_list_col)) {
-    stopc("Can't use list columns in `frame_matrix()`")
+  list_cols <- which(map_lgl(rest, needs_list_col))
+  if (has_length(list_cols)) {
+    cnd_signal(error_frame_matrix_list(list_cols))
   }
 
   frame_ncol <- length(names)
@@ -162,3 +175,39 @@ turn_frame_data_into_frame_matrix <- function(names, rest) {
   colnames(frame_mat) <- names
   frame_mat
 }
+
+# Errors ------------------------------------------------------------------
+
+error_tribble_needs_columns <- function() {
+  tibble_error("Must specify at least one column using the `~name` syntax.")
+}
+
+error_tribble_lhs_column_syntax <- function(lhs) {
+  tibble_error(bullets(
+    "All column specifications must use the `~name` syntax.",
+    paste0("Found ", expr_label(lhs), " on the left-hand side of `~`.")
+  ))
+}
+
+error_tribble_rhs_column_syntax <- function(rhs) {
+  tibble_error(bullets(
+    'All column specifications must use the `~name` or `~"name"` syntax.',
+    paste0("Found ", expr_label(rhs), " on the right-hand side of `~`.")
+  ))
+}
+
+error_tribble_non_rectangular <- function(cols, cells) {
+  tibble_error(bullets(
+    "Data must be rectangular:",
+    paste0("Found ", cols, " columns."),
+    paste0("Found ", cells, " cells."),
+    i = paste0(cells, " is not an integer multiple of ", cols, ".")
+  ))
+}
+
+error_frame_matrix_list <- function(pos) {
+  tibble_error(bullets(
+    "All values must be atomic:",
+    pluralise_commas("Found list-valued element(s) at position(s) ", pos, ".")
+  ))
+}
