@@ -284,6 +284,14 @@ test_that("[.tbl_df ignores drop argument (with warning) without j argument (#30
   expect_warning(expect_identical(df_all[1, drop = TRUE], df_all[1]))
 })
 
+test_that("[.tbl_df emits errors with matrix row subsetting (#760)", {
+  scoped_lifecycle_errors()
+
+  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
+  expect_error(foo[matrix(1:2, ncol = 2), ])
+  expect_error(foo[matrix(rep(TRUE, 10), ncol = 2), ])
+})
+
 
 test_that("[.tbl_df is careful about attributes (#155)", {
   df <- tibble(x = 1:2, y = x)
@@ -518,6 +526,38 @@ test_that("[<-.tbl_df supports adding duplicate columns", {
   expect_identical(df, tibble(x = 1:2, x = 3:4, .name_repair = "minimal"))
 })
 
+
+test_that("[<-.tbl_df supports matrix on the RHS (#762)", {
+  df <- tibble(x = 1:4, y = letters[1:4])
+  df[1:2] <- matrix(8:1, ncol = 2)
+  expect_identical(df, tibble(x = 8:5, y = 4:1))
+
+  df <- tibble(x = 1:4, y = letters[1:4])
+  df[1:2] <- array(4:1, dim = c(4, 1, 1))
+  expect_identical(df, tibble(x = 4:1, y = 4:1))
+
+  df <- tibble(x = 1:4, y = letters[1:4])
+  df[1:2] <- array(8:1, dim = c(4, 2, 1))
+  expect_identical(df, tibble(x = 8:5, y = 4:1))
+
+  df <- tibble(x = 1:4, y = letters[1:4])
+  expect_tibble_error(
+    df[1:3, 1:2] <- matrix(6:1, ncol = 2),
+    error_assign_incompatible_type(
+      df, as.data.frame(matrix(6:1, ncol = 2)), 2, quote(matrix(6:1, ncol = 2)),
+      cnd_message(tryCatch(vctrs::vec_assign(letters, 1:3, 3:1), error = identity))
+    )
+  )
+  expect_tibble_error(
+    df[1:2] <- array(8:1, dim = c(2, 1, 4)),
+    error_need_rhs_vector_or_null(quote(array(8:1, dim = c(2, 1, 4))))
+  )
+  expect_tibble_error(
+    df[1:2] <- array(8:1, dim = c(4, 1, 2)),
+    error_need_rhs_vector_or_null(quote(array(8:1, dim = c(4, 1, 2))))
+  )
+})
+
 test_that("[<- with explicit NULL doesn't change anything (#696)", {
   iris_tbl_orig <- as_tibble(iris)
 
@@ -691,7 +731,6 @@ verify_output("subsetting.txt", {
   foo[c(-1, 1), ]
   foo[c(-1, NA), ]
   invisible(foo[-4, ])
-  foo[as.matrix(1), ]
   foo[array(1, dim = c(1, 1, 1)), ]
   foo[mean, ]
   foo[foo, ]
@@ -708,7 +747,6 @@ verify_output("subsetting.txt", {
   foo <- tibble(x = 1:3, y = 1:3, z = 1:3)
   foo[c(TRUE, TRUE), ]
   foo[c(TRUE, TRUE, FALSE, FALSE), ]
-  foo[as.matrix(TRUE), ]
   foo[array(TRUE, dim = c(1, 1, 1)), ]
 
   "# [.tbl_df rejects unknown column indexes (#83)"
@@ -770,6 +808,11 @@ verify_output("subsetting.txt", {
   foo[factor(1:3)] <- 1
   foo[Sys.Date()] <- 1
 
+  "# [.tbl_df emits lifecycle warnings with one-column matrix indexes (#760)"
+  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
+  invisible(foo[matrix(1:2, ncol = 1), ])
+  invisible(foo[matrix(rep(TRUE, 10), ncol = 1), ])
+
   "# [<-.tbl_df rejects unknown row indexes"
   foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
   foo[list(1:3), ] <- 1
@@ -804,6 +847,7 @@ verify_output("subsetting.txt", {
   df <- tibble(x = 1:3, y = x, z = y)
   df[1:2] <- list(0, 0, 0)
   df[] <- list(0, 0)
+  df[1, ] <- 1:3
   df[1:2, ] <- 1:3
   df[,] <- 1:2
 
